import QtQuick 1.0
import com.meego 1.0
import "."
import Metronome 1.0
import QtMultimediaKit 1.1
import "metronome.js" as Helpers

Page {
     id: mainpage
     signal itemActivated

     property int dialWidth: 418
     property int dialTopMargin: 100
     property int dialLeftMargin: 26
     property int startBottomMargin: 70

     orientationLock: 1

     // pause metronome when backgrounded
     Connections {
         target: screen.windowState
         onActiveChanged: {
             if (screen.minimized || !screen.windowState.active) {
                 mainpage.state = "tickOff"
             }
         }
     }

     SoundEffect {
         id: clickSound
         source: "/opt/metronome/metronome_snd_004.wav"
     }

     Metronome {
         id: metro
         onTick: {
             if (!metro.muteOn) {
                 clickSound.play();
             }
             tickAni.start();
         }
     }

    Image {
        id: bg
        width: sourceSize.width
        height: sourceSize.height
        smooth: true
        source: "Background_04.jpg"
    }

    Image {
        id: dial
        width: dialWidth
        height: width
        anchors.top: parent.top
        anchors.topMargin: dialTopMargin
        anchors.horizontalCenter: parent.horizontalCenter
        smooth: true
        source: "ring02.png"
        MouseArea {
            anchors.fill: parent
	    anchors.topMargin: -15
	    anchors.rightMargin: -15
	    anchors.leftMargin: -15
            onClicked: {
                var angle = Helpers.mapToDial(mouse.x, mouse.y);
                if (angle >= 110) {
                    bpmAnimation.to = Helpers.angleToBpm(angle)
                    bpmAnimation.running = true
                }
                else {
                    // Manual click handling
                    //if (!metro.metroOn) {
                    if (true) {
                        var avgTap = Helpers.getAverageTap();
			if (avgTap >= 30 && avgTap <= 230)
			   metro.bpm = avgTap;
                        tickAni.start()
                    }
                }
            }
            onPositionChanged: {
                var angle = Helpers.mapToDial(mouse.x, mouse.y);
                if (angle >= 110)
                    metro.bpm = Helpers.angleToBpm(angle)
            }
        }
    }

    Repeater {
        model: 37
        Image {
            x: parent.width/2
            y: 310
            source: "ring01segment2.png"
            smooth: true
            visible: knobRotation.angle > 200 + index*8.64
            transform: [
                Rotation {
                    id: rota
                    angle: 180*0.145/Math.PI*index - 180 + 30
                },
                Translate {
                    y: 203*(Math.sin(0.63*Math.PI + 0.145*index))
                    x: 203*(Math.cos(0.63*Math.PI + 0.145*index))
                }
            ]
        }
    }

    Image {
        id: led
        width: dial.width
        height: width
        anchors.top:  dial.top
        anchors.topMargin: 2
        anchors.left:  dial.left
        anchors.leftMargin: -2
        opacity: 0
        //visible: metro.metroOn
        SequentialAnimation {
            id: tickAni
            NumberAnimation { target: led; property: "opacity"; from: 0.8; to: 0; duration: 50 }
        }
        source: "led.png"
    }

    Image {
        id: knob
        x: 218
        y: dialTopMargin - 9
        smooth: true
        source: "marker.png"
        //visible: !metro.metroOn
        transform: Rotation {
            id: knobRotation
            origin.x: 20
            origin.y: 218
            angle: Helpers.bpmToAngle(metro.bpm) + 85
        }
     }
    Text {
        id: bpmText
        anchors.horizontalCenter: dial.horizontalCenter
        anchors.verticalCenter: dial.verticalCenter
        text: metro.bpm
        font.bold: true
        font.pointSize: 0.25*dial.height;
        color: "white"
    }

    Image {
        id: startButton
        width: sourceSize.Width
        height: sourceSite.height
        anchors.horizontalCenter: parent.horizontalCenter

        anchors.bottom: parent.bottom
        anchors.bottomMargin: startBottomMargin
        smooth: true
        source: {metro.metroOn ? "Pause_Button_24bit_03.png" : "Play_Button_24bit_03.png"}
        //source: {metro.metroOn ? "metro-pause-button.png" : "metro-play-button.png"}

        MouseArea {
            anchors.fill: parent
            anchors.topMargin: -70
            anchors.leftMargin: -100
            anchors.rightMargin: -100
            anchors.bottomMargin: -70
            onClicked: {
                if (metro.metroOn == true) {
                    mainpage.state = "tickOff"
                }
                else {
                    mainpage.state = "tickOn"
                }
            }
        }
    }

    Image {
        id: muteButton
        anchors.top: parent.top
	anchors.topMargin: 35
	anchors.right: parent.right
	anchors.rightMargin: 43
        smooth: true
        source: {metro.muteOn ? "mute-icon-on.png" : "mute-icon-off.png"}

        MouseArea {
            anchors.fill: parent
            anchors.topMargin: -30
            anchors.leftMargin: -30
            anchors.rightMargin: -30
            onClicked: {
                metro.muteOn = !metro.muteOn;
            }
        }
    }

    PropertyAnimation {
        id: bpmAnimation
        target: metro;
        properties: "bpm"
        duration: 200
    }
    Button {
        id: plusButton
        width:  50
        height:  width
        anchors.top: dial.bottom
        anchors.topMargin: -20
        anchors.right: parent.right
        anchors.rightMargin: 10
        buttonText: "+"
        onClicked: {
            if (metro.bpm < 230)
                metro.bpm = metro.bpm + 1
        }
        onPressedRepeat: {
            var modulo = 10 - (metro.bpm % 10);
            bpmAnimation.to = Math.min(230, metro.bpm + modulo);
            bpmAnimation.running = true
        }
    }

    Button {
        id: minusButton
        width:  50
        height:  width
        anchors.top: dial.bottom
        anchors.topMargin: -20
        anchors.left: parent.left
        anchors.leftMargin: 10
        buttonText: "-"
        onClicked: {
            if (metro.bpm > 30)
                metro.bpm = metro.bpm - 1
        }
        onPressedRepeat: {
            var modulo = (metro.bpm % 10);
            if (modulo == 0) modulo = 10;
            bpmAnimation.to = Math.max(30, metro.bpm - modulo);
            bpmAnimation.running = true
        }
    }

    states: [
        State {
            name: "tickOn"
            PropertyChanges { target: metro; metroOn: true }
        },

        State {
            name: "tickOff"
            PropertyChanges { target: metro; metroOn: false }
        }
    ]
}
